"""
by Dominic Pezzuto
"""

import GameStarter
import pygame
import time
from random import *

lazerSprites = pygame.sprite.Group()
lazerNumPoints = 200


activeSprites = pygame.sprite.Group()
enemies = pygame.sprite.Group()
bullets = pygame.sprite.Group()

class Pattern:
    def __init__(self):
        self.movement = []
        self.repeats = {}

pattern1 = Pattern()
pattern1.movement = [ [-1,0], [0,0], [1,0], [0,0] ]
pattern1.repeats[0] = 20
pattern1.repeats[1] = 30
pattern1.repeats[2] = 20
pattern1.repeats[3] = 30

patternDL = Pattern()
patternDL.movement = [ [-1, 1] ]
patternDL.repeats[0] = 500

patternDR = Pattern()
patternDR.movement = [ [1, 1] ]
patternDR.repeats[0] = 500

patternUL = Pattern()
patternUL.movement = [ [-1, -1] ]
patternUL.repeats[0] = 500

patternUR = Pattern()
patternUR.movement = [ [1, -1] ]
patternUR.repeats[0] = 500

patternDFast = Pattern()
patternDFast.movement = [ [0, 3] ]
patternDFast.repeats[0] = 500

patternDReg = Pattern()
patternDReg.movement = [ [0, 1] ]
patternDReg.repeats[0] = 500

patternDSlow = Pattern()
patternDSlow.movement = [ [0, 1], [0, 0] ]
patternDSlow.repeats[0] = 1
patternDSlow.repeats[1] = 3

class mySprite(GameStarter.Sprite):
    def __init__(self, image=None, frameset = None):
        super(mySprite, self).__init__(image)
        if frameset!=None:
            self.frames = frameset

        self.active = True

    def incrementColor(self, d):
        if self.image != None:
            ignore = list(self.image.get_colorkey())
            for y in range(self.image.get_height()):
                for x in range(self.image.get_width()):
                    color = list(self.image.get_at( (x,y) ))
                    if color!=ignore:
                        for i in range(3):
                            if color[i] < 255 - d:
                                color[i] += d
                        self.image.set_at( (x,y), color)

    
    

class Player(mySprite):

    lazerPoints = [ [0,0], [1,1] ]
    lazerOffset = [0,0]
    lazerCurveMidpointOffset = [0,0]    
    

    def __init__(self, image=None, frameset = None, startpos = None):
        super(Player, self).__init__(image, frameset)
        self.moveSpeed = [5,5]
        self.currentFrameset = "idle"
        if startpos != None:
            self.rect[0] = startpos[0]
            self.rect[1] = startpos[1]
        self.lazerType = "Normal"
        self.lazerDamage = 2
        self.lazerWidth = 10
        self.lazerWidthMax = 10
        self.lazerColor = [200, 200, 200]
        self.colorDec = 5
        self.colorDir = -1


    def buildLazer(self):
        global lazerSprites, lazerNumPoints
        lazerSprites.empty()
        
        midpoint = (self.rect[0] + self.image.get_width()/2)
        self.lazerPoints[0] = [midpoint, self.rect[1]]
        self.lazerPoints[-1] = [midpoint+ self.lazerOffset[0], 0]

        self.lazerPoints = self.getLazerCurve(lazerNumPoints)

        beg = time.time()
        scount = 0
        for i in self.lazerPoints:
            if scount == lazerNumPoints / 50:
                scount = 0
                stemp = pygame.sprite.Sprite()
                stemp.rect = pygame.Rect(i[0]-self.lazerWidth/2, i[1]-self.lazerWidth/2, self.lazerWidth, self.lazerWidth)
                lazerSprites.add(stemp)
            scount += 1

    def drawLazer(self):
        color = self.lazerColor
##        pygame.draw.lines(GameStarter.screen, (255, 255, 255), False, self.lazerPoints, self.lazerWidth)
        for i in self.lazerPoints:
            pygame.draw.circle(GameStarter.screen, color, i, self.lazerWidth/2)
##            if color[0] <= 0:
##                color[0] = 255
##            else:
##                color[0] -= 1

        if color[1] <= self.colorDec or color[1] >= self.colorDec + 200:
##            color[1] = 200
            self.colorDir = -1 * self.colorDir
            color[1] += (self.colorDir * self.colorDec)
        else:
            color[1] += (self.colorDir * self.colorDec)

        if color[2] <= self.colorDec or color[2] >= self.colorDec + 200:
##            color[2] = 200
##            self.colorDir = -1 * self.colorDir
            color[2] += (self.colorDir * self.colorDec)
            print color
        else:
            color[2] += (self.colorDir * self.colorDec)

##        for i in range(len(self.lazerPoints)-1):
##            pygame.draw.line(GameStarter.screen, (255, 255, 255), self.lazerPoints[i], self.lazerPoints[i+1], self.lazerWidth)
            
        
    def getLazerCurvePoint(self, u, p0i, p1i, p2i):      
        full= [0.0,0.0]
        
        p0 = p0i[0]
        p1 = p1i[0]
        p2 = p2i[0]        
        result = p0 * pow((1-u),2)
        result +=  p1 * (2*u)*(1-u)
        result +=  p2 * pow(u,2)
        full[0] = int(result)

        p0 = p0i[1]
        p1 = p1i[1]
        p2 = p2i[1]        
        result = p0 * pow((1-u),2)
        result +=  p1 * (2*u)*(1-u)
        result +=  p2 * pow(u,2)
        full[1] = int(result)
        
        return full

    def getLazerCurve(self, numpoints):
        first = self.lazerPoints[0]
        last = self.lazerPoints[-1]

        midpointX = (self.rect[0] + self.image.get_width()/2)
        midpointY = (self.rect[1] / 2) 
        
        mid = [midpointX + self.lazerCurveMidpointOffset[0], midpointY + self.lazerCurveMidpointOffset[1]]


        
        result = []

        x = 0.0
        inc = float(1)/float(numpoints)
        
        for i in range(numpoints + 1):
            result += [self.getLazerCurvePoint(x, first, mid, last)]
            x += inc

        return result


class Enemy(mySprite):
    global pattern1
    def __init__(self, startpos, pattern = None, hp = 100, frameset = None, bulletFrameset = None):
        super(Enemy, self).__init__(None, frameset)
        self.movePattern = pattern
        self.repeatCounter = 0
        self.patternPosition = 0
        self.health = hp
        self.rect[0] = float(startpos[0])
        self.rect[1] = float(startpos[1])
        self.currentFrameset = "idle"
        self.killValue = 200
        self.bulletFrameset = bulletFrameset
        self.bulletTimer = 0
        self.bulletLimit = 2
        self.whiteframes = {}
        self.whiteframes["idle"] = pygame.Surface(( 200, 200 ))
        self.whiteframes["idle"].fill( (255, 255, 255) )
        self.whitealpha = 0


    def destroy ( self ):
        self.leaveItems()
        self.kill()


    def update( self ):
        super(Enemy, self).update()

        #self.whiteframes["idle"].set_alpha(self.whitealpha)
        #self.image.blit(self.whiteframes["idle"],(0,0))

        #Move according to given pattern
        if self.movePattern!=None:
            self.moveSpaces(self.movePattern.movement[self.patternPosition][0],self.movePattern.movement[self.patternPosition][1])
            if self.patternPosition in self.movePattern.repeats.keys():
                self.repeatCounter += 1
                if self.repeatCounter >= self.movePattern.repeats[self.patternPosition]:
                    self.patternPosition += 1
                    self.repeatCounter = 0                
            else:
                self.patternPosition += 1
            if self.patternPosition >= len(self.movePattern.movement):
                self.patternPosition = 0
        else:
            self.altMove()
        
##        self.rect[2] = self.image.get_width()
##        self.rect[3] = self.image.get_height()        
##        self.animate( self.currentFrameset )

        #If enemy goes too far off screen, kill it
        if (self.rect[0] > GameStarter.screen.get_width() + self.killValue) or\
           (self.rect[0] < -self.killValue) or\
           (self.rect[1] > GameStarter.screen.get_height() + self.killValue) or\
           (self.rect[1] < -self.killValue):
            print "bang"
            self.kill()

        if self.bulletTimer >= self.bulletLimit:
            self.bulletTimer = 0
            self.shootBullet()
        else:
            self.bulletTimer += 1


    def shootBullet( self ):
        pass


    def altMove(self):
        pass

    def leaveItems( self ):
        pass


######################
## Enemy Subclasses ##
######################
    
class FatShip(Enemy):
    def __init__(self, startpos, pattern = None, hp = 100, frameset = None):
        super(FatShip, self).__init__(startpos, pattern=pattern, hp = hp, frameset=frameset)
        self.moveSpeed = [1, 1]

        
class Zipper(Enemy):
    def __init__(self, startpos, pattern = None, hp = 17, frameset = None, bulletFrameset = None):
        super(Zipper, self).__init__(startpos, pattern=pattern, hp = hp, frameset=frameset, bulletFrameset = bulletFrameset)
        self.moveSpeed = [6, 6]
        self.bulletLimit = 10

    def shootBullet( self ):
	if( randint(1,4) == 1 ):
	    return



	randA = randint(2,4)
	randB = randint(2,4)

	randC = 0
	while randC == 0:
            randC = randint(-1,1)

	randD = 0
	while randD == 0:
            randD = randint(-1,1)

	randA = randA * randC
	randB = randB * randD

        bullets.add( Bullet( self.rect, pattern = patternDReg, frameset = self.bulletFrameset, speed = [randA,randB] ) )


class Axehead(Enemy):
    def __init__(self, startpos, pattern = None, hp = 17, frameset = None):
        super(Axehead, self).__init__(startpos, pattern=pattern, hp = hp, frameset=frameset)
        self.moveSpeed = [2, 2]

        
#############################
## End of Enemy Subclasses ##
#############################



class Bullet(Enemy):
    def __init__(self, startpos, pattern = None, hp = 17, frameset = None, speed = [1,1]):
        super(Bullet, self).__init__(startpos, pattern=pattern, hp = hp, frameset=frameset)        
##        super(Bullet, self).__init__(None, frameset)
        self.moveSpeed = speed
        self.currentFrameset = "idle"


class PowerUp(mySprite):
    pass


class Particle():
    def __init__(self, coords, size = 1.0, color = [100,100,100]):
        self.coords = coords
        self.startColor = color
        self.color = color
        self.startSize = size
        self.size = size

    def expand( self ):
        pass

    def changeColor( self ):
        pass

    def move( self ):
        pass

    def go( self ):
        self.expand()
        self.changeColor()
        self.move()
        
class Particle1( Particle ):
    def __init__(self, coords, size = 1.0, color = [100,100,100]):
        super(Particle1, self).__init__(coords, size = size, color = color)
        self.dsize = 1
        self.maxsize = 50.0
        self.minsize = 0.0
        
    def expand( self ):
        if not ( ( ( self.dsize + self.size ) < self.maxsize ) and ( ( self.dsize + self.size ) > self.minsize ) ):
            self.dsize *= -1

        self.size += self.dsize
    
    

class Explosion( mySprite ):
    def __init__(self, image=None, frameset = None, startpos = None):
        super(Explosion, self).__init__(image, frameset)
        self.particles = []
        
    def explode( self ):
        for particle in self.particles:
            particle.go()

    
        

##class Point
